#!/bin/sh
#
# @(#) create_rel_br
#
# Usage:
#   create_rel_br
# Description
#   create and push release branch and tag.
# Return
#   0: Success
#   not 0: Fail
# ./create_rel_br.sh -v 1.00_9 -x

progname=${0##*/}
progdir=${0%/*}
export LANG=C

# 使用方法
usage() {
    cat <<EOF
$progname [-v ver] [-x debug]
    -v ver バージョン指定。
        ?.??_? 形式。
    -x debug用表示追加
    releaseブランチの作成とversion.dat, specファイルのversion/dateを更新する。
EOF
    exit 2
}

# オプションからパラメータを生成する
Verbose=0
while getopts v:x OPT; do
    case ${OPT} in
    v) Version="${OPTARG}" ;;
    x) Verbose=1 ;;
    *) usage ;;
    esac
done
shift $((OPTIND - 1))

get_options() {
    # fix parameters
    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<cehck parameters>"
    fi
    if [[ ! "${Version}" =~ ^[0-9]*.[0-9]*_[0-9a-zA-Z]*$ ]]; then
        echo " - Error: Input valid version ?.??_?"
        exit 1
    fi

    Tag=V$(echo "${Version}" | sed 's/\.//g')
    Rel_branch="release/${Tag}"

    Ver_file=$(find -maxdepth 3 -name version.dat -type f)

    Spec_file=$(find -maxdepth 1 -name \*.spec -type f)
    if [ ${#Spec_file[@]} -ne 1 ]; then
        echo " - Error: Not found *.spec"
        exit 1
    fi
    if ! [ -f "${Spec_file}" ]; then
        echo " - Error: Not found *.spec"
        exit 1
    fi

    Timestamp=$(env LC_ALL=C TZ=Asia/Tokyo date '+%a %b %d %Y_%Y-%m-%d %H:%M')
    Message="Release ${Version} "$(echo -n "${Timestamp}" | cut -f 2 -d "_")

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<parameters>"
        echo " - version: ${Version}"
        echo " - tag: ${Tag}"
        echo " - rel_branch: ${Rel_branch}"
        for file in ${Ver_file[@]}; do
            echo " - ver_file: ${file}"
        done
        echo " - spec_file: ${Spec_file}"
        echo " - timestamp: ${Timestamp}"
        echo " - message: ${Message}"
    fi
}

# Master Branch以外であれば、Master Branchに戻る
return_master_br() {
    local rv
    rv=0

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git pull>"
    fi
    git pull

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git cehckout master>"
    fi
    git checkout master
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git checkout master"
        exit 1
    fi
    if [ "master" != $(git rev-parse --abbrev-ref HEAD) ]; then
        echo " - Error: git checkout master"
        exit 1
    fi

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git submodule update>"
    fi
    git submodule update
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git submodule update"
        exit 1
    fi

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git checkout HEAD>"
    fi

    git checkout HEAD "${Spec_file}"
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git checkout HEAD ${Spec_file}"
        exit 1
    fi
    for file in ${Ver_file[@]}; do
        git checkout HEAD "${file}"
        rv=$?
        if [ ${rv} -ne 0 ]; then
            echo " - Error: git checkout HEAD ${file}"
            exit 1
        fi
    done
}

# Version/Branch/Tagの二重定義を許可しない
verify_double_define() {
    local old_version
    local old_branch
    local old_tag

    old_version=$(grep -E "^Version:" ${Spec_file} | sed -e "s/^Version: *\(.*\)$/\1/")
    if [ "${Version}" = "${old_version}" ]; then
        echo " - Error: [Version] ${Version} has been already used."
        exit 1
    fi

    old_branch=$(git branch -a | grep ${Rel_branch})
    for old in ${old_branch[@]}; do
        if [ "${old}" ]; then
            echo " - Error: [Branch] ${Rel_branch} has been already used. Please delete ${Rel_branch}."
            echo "   -> git branch -D ${Rel_branch}"
            echo "   -> git push --delete origin ${Rel_branch}"
            exit 1
        fi
    done

    old_tag=$(git tag | grep -E "${Tag}$")
    for old in ${old_tag[@]}; do
        if [ "${old}" ]; then
            echo " - Error: Tag [${Tag}] has been already used. Please delete ${Tag}."
            echo " - [Caution] 過去のリリースで使用したTagを削除してはいけません。"
            echo "   -> git tag -d ${Tag}"
            echo "   -> git push --delete origin ${Tag}"
            exit 1
        fi
    done
}

# Release BranchをCheck Outする
checkout_release_br() {
    local rv
    rv=0

    git checkout -b "${Rel_branch}"
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git checkout -b ${Rel_branch}"
        exit 1
    fi

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<modify spec and version.dat file>"
    fi
}

# specファイルとversion.datファイルを更新する
modify_file() {
    local change_log
    local module

    # modify spec file
    change_log="\n* $(echo ${Timestamp} | cut -f 1 -d '_') ULM Project <ulm-soft@vsp.cpg.sony.co.jp> - ${Version}\n- Release Version: ${Version}\n"
    sed -i -e "s/^\(Version: *\).*$/\1${Version}/g" "${Spec_file}"
    sed -i -e "s/^\(%changelog\)$/\1$change_log/g" "${Spec_file}"

    # modify version.dat file
    for file in ${Ver_file[@]}; do
        module=$(cat ${file} | cut -f 1 -d " ")
        echo -n "${module} ${Version} " >"${file}"
        echo -n "${Timestamp}" | cut -f 2 -d "_" >>"${file}"
    done
}

# git add
git_add() {
    local rv
    rv=0

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git add ${Spec_file} ${Ver_file}>"
    fi
    git add ${Spec_file}
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git add ${Spec_file}"
        exit 1
    fi
    for file in ${Ver_file[@]}; do
        git add "${file}"
        rv=$?
        if [ ${rv} -ne 0 ]; then
            echo " - Error: git add ${file}"
            exit 1
        fi
    done
}

# git commit
git_commit() {
    local rv
    rv=0

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git commit>"
    fi
    git commit -m "${Message}"
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git commit -m ${Message}"
        exit 1
    fi
}

# git add tag
git_add_tag() {
    local rv
    rv=0

    # tag
    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git tag>"
    fi
    git tag -a "${Tag}" -m "${Message}"
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git tag -a ${Tag} -m ${Message}"
        exit 1
    fi
}

# git push
git_push() {
    local rv
    rv=0

    if [ ${Verbose} -eq 1 ]; then
        echo -e "\n<git push>"
    fi
    git push origin "${Rel_branch}" "${Tag}"
    rv=$?
    if [ ${rv} -ne 0 ]; then
        echo " - Error: git push origin ${Rel_branch} ${Tag}"
        exit 1
    fi
}

get_options
return_master_br
verify_double_define
checkout_release_br
modify_file
git_add
git_commit
git_add_tag
git_push

echo
echo "Success!!"
exit 0
